#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/classifier/GaussianProcessClassification.h>
#include <shogun/machine/gp/SoftMaxLikelihood.h>
#include <shogun/mathematics/Math.h>
#include <shogun/machine/gp/ConstMean.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/machine/gp/MultiLaplaceInferenceMethod.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/io/CSVFile.h>
#include <shogun/evaluation/MulticlassAccuracy.h>
#include <shogun/lib/SGVector.h>
#include <shogun/kernel/GaussianKernel.h>
#include <shogun/labels/MulticlassLabels.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/classifier_4class_2d_linear_labels_test.dat");
CMath::init_random(1);

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CMulticlassLabels>(f_labels_train);
auto labels_test = some<CMulticlassLabels>(f_labels_test);
//![create_features]

//![create_appropriate_kernel_and_mean_function]
auto kernel = some<CGaussianKernel>(2.0);
auto mean_function = some<CConstMean>();
//![create_appropriate_kernel_and_mean_function]

//![create_instance]
auto gauss_likelihood = some<CSoftMaxLikelihood>();
auto inference_method = some<CMultiLaplaceInferenceMethod>(kernel, features_train, mean_function, labels_train, gauss_likelihood);
auto gp_classifier = some<CGaussianProcessClassification>(inference_method);
//![create_instance]

//![train_and_apply]
gp_classifier->train();
auto labels_predict = gp_classifier->apply_multiclass(features_test);
//![train_and_apply]

//![extract_the_probabilities]
auto prob = gp_classifier->get_probabilities(features_train);
//![extract_the_probabilities]

//![evaluate_accuracy]
auto evals = some<CMulticlassAccuracy>();
auto accuracy = evals->evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// integration testing variables
auto output = labels_test->get_labels();

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("gaussian_process_classifier.dat", 'w');
__sg_storage->append_wrapped_real_vector(prob, "prob");
__sg_storage->append_wrapped_real(accuracy, "accuracy");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
