#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/lib/SGVector.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/multiclass/GaussianNaiveBayes.h>
#include <shogun/io/CSVFile.h>
#include <shogun/labels/MulticlassLabels.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_labels_train.dat");

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CMulticlassLabels>(f_labels_train);
//![create_features]


//![create_instance]
auto gnb = some<CGaussianNaiveBayes>(features_train, labels_train);
//![create_instance]

//![train_and_apply]
gnb->train();
auto labels_predict = gnb->apply_multiclass(features_test);
//![train_and_apply]


// integration testing variables
auto output = labels_predict->get_labels();

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("gaussian_naive_bayes.dat", 'w');
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
