#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/multiclass/MCLDA.h>
#include <shogun/labels/MulticlassLabels.h>
#include <shogun/evaluation/MulticlassAccuracy.h>
#include <shogun/lib/SGVector.h>
#include <shogun/io/CSVFile.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/lib/SGMatrix.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/classifier_4class_2d_linear_labels_test.dat");

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CMulticlassLabels>(f_labels_train);
auto labels_test = some<CMulticlassLabels>(f_labels_test);
//![create_features]

//![create_instance]
auto mc_lda = some<CMCLDA>(features_train, labels_train, 0.0001, true);
//![create_instance]

//![train_and_apply]
mc_lda->train();
auto labels_predict = mc_lda->apply_multiclass(features_test);
//![train_and_apply]

//![extract_mean_and_cov]
auto classlabel = 1;
auto m = mc_lda->get_mean(classlabel);
auto c = mc_lda->get_cov();
//![extract_mean_and_cov]

//![evaluate_accuracy]
auto evals = some<CMulticlassAccuracy>();
auto accuracy = evals->evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// additional integration testing variables
auto output = labels_predict->get_labels();

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("linear_discriminant_analysis.dat", 'w');
__sg_storage->append_wrapped_int(classlabel, "classlabel");
__sg_storage->append_wrapped_real_vector(m, "m");
__sg_storage->append_wrapped_real_matrix(c, "c");
__sg_storage->append_wrapped_real(accuracy, "accuracy");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
