#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/multiclass/tree/RelaxedTree.h>
#include <shogun/io/CSVFile.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/kernel/GaussianKernel.h>
#include <shogun/lib/SGVector.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/evaluation/MulticlassAccuracy.h>
#include <shogun/labels/MulticlassLabels.h>
#include <shogun/multiclass/MulticlassLibLinear.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/classifier_4class_2d_linear_labels_test.dat");

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CMulticlassLabels>(f_labels_train);
auto labels_test = some<CMulticlassLabels>(f_labels_test);
//![create_features]

//![set_parameters]
auto mll = some<CMulticlassLibLinear>();
auto kernel = some<CGaussianKernel>();
//![set_parameters]

//![create_instance]
auto machine = some<CRelaxedTree>();
machine->set_labels(labels_train);
machine->set_machine_for_confusion_matrix(mll);
machine->set_kernel(kernel);
//![create_instance]

//![train_and_apply]
machine->train(features_train);
auto labels_predict = machine->apply_multiclass(features_test);
//![train_and_apply]

//![evaluate_accuracy]
auto eval = some<CMulticlassAccuracy>();
auto accuracy = eval->evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// integration testing variables
auto output = labels_predict->get_labels();

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("relaxed_tree.dat", 'w');
__sg_storage->append_wrapped_real(accuracy, "accuracy");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
