#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/preprocessor/PruneVarSubMean.h>
#include <shogun/evaluation/MeanSquaredError.h>
#include <shogun/io/CSVFile.h>
#include <shogun/labels/RegressionLabels.h>
#include <shogun/lib/SGVector.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/preprocessor/NormOne.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/regression/LeastAngleRegression.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/regression_1d_linear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/regression_1d_linear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/regression_1d_linear_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/regression_1d_linear_labels_test.dat");

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CRegressionLabels>(f_labels_train);
auto labels_test = some<CRegressionLabels>(f_labels_test);
//![create_features]

//![preprocess_features]
auto SubMean = some<CPruneVarSubMean>();
auto Normalize = some<CNormOne>();
SubMean->init(features_train);
SubMean->apply_to_feature_matrix(features_train);
SubMean->apply_to_feature_matrix(features_test);
Normalize->init(features_train);
Normalize->apply_to_feature_matrix(features_train);
Normalize->apply_to_feature_matrix(features_test);
//![preprocess_features]

//![create_instance]
auto lamda1 = 0.01;
auto lars = some<CLeastAngleRegression>(false);
lars->set_features(features_train);
lars->set_labels(labels_train);
lars->set_max_l1_norm(lamda1);
//![create_instance]

//![train_and_apply]
lars->train();
auto labels_predict = lars->apply_regression(features_test);

//[!extract_w]
auto weights = lars->get_w();
//[!extract_w]

//![evaluate_error]
auto eval = some<CMeanSquaredError>();
auto mse = eval->evaluate(labels_predict, labels_test);
//![evaluate_error]

// integration testing variables
auto output = labels_test->get_labels();


// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("least_angle_regression.dat", 'w');
__sg_storage->append_wrapped_real(lamda1, "lamda1");
__sg_storage->append_wrapped_real_vector(weights, "weights");
__sg_storage->append_wrapped_real(mse, "mse");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
