#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/ensemble/MeanRule.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/labels/RegressionLabels.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/evaluation/MeanSquaredError.h>
#include <shogun/machine/RandomForest.h>
#include <shogun/io/CSVFile.h>
#include <shogun/lib/SGVector.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/mathematics/Math.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/regression_1d_linear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/regression_1d_linear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/regression_1d_linear_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/regression_1d_linear_labels_test.dat");
CMath::init_random(1);

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CRegressionLabels>(f_labels_train);
auto labels_test = some<CRegressionLabels>(f_labels_test);
//![create_features]

//![create_combination_rule]
auto mean_rule = some<CMeanRule>();
//![create_combination_rule]

//![create_instance]
auto rand_forest = some<CRandomForest>(features_train, labels_train, 5);
rand_forest->set_combination_rule(mean_rule);
//![create_instance]

//![train_and_apply]
rand_forest->train();
auto labels_predict = rand_forest->apply_regression(features_test);
//![train_and_apply]

//![evaluate_error]
auto mse = some<CMeanSquaredError>();
auto oob = rand_forest->get_oob_error(mse);
auto mserror = mse->evaluate(labels_predict, labels_test);
//![evaluate_error]

// additional integration testing variables
auto output = labels_predict->get_labels();

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("random_forest_regression.dat", 'w');
__sg_storage->append_wrapped_real(oob, "oob");
__sg_storage->append_wrapped_real(mserror, "mserror");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
