#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/evaluation/MeanSquaredError.h>
#include <shogun/lib/SGVector.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/io/CSVFile.h>
#include <shogun/labels/RegressionLabels.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/kernel/GaussianKernel.h>
#include <shogun/regression/svr/LibSVR.h>
#include <shogun/lib/WrappedObjectArray.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/regression_1d_sinc_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/regression_1d_sinc_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/regression_1d_sinc_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/regression_1d_sinc_labels_test.dat");

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CRegressionLabels>(f_labels_train);
auto labels_test = some<CRegressionLabels>(f_labels_test);
//![create_features]

//![create_appropriate_kernel]
auto width = 1.0;
auto kernel = some<CGaussianKernel>(width);
//![create_appropriate_kernel]

//![create_instance]
auto svm_c = 1.0;
auto svr_param = 0.1;
auto svr = some<CLibSVR>(svm_c, svr_param, kernel, labels_train, LIBSVR_SOLVER_TYPE::LIBSVR_EPSILON_SVR);
//![create_instance]

//![train_and_apply]
svr->train(features_train);
auto labels_predict = svr->apply_regression(features_test);
//![train_and_apply]

//![extract_alpha]
auto alpha = svr->get_alphas();
//![extract_alpha]

//![evaluate_error]
auto eval = some<CMeanSquaredError>();
auto mse = eval->evaluate(labels_predict, labels_test);
//![evaluate_error]

// integration testing variables
auto output = labels_test->get_labels();

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("support_vector_regression.dat", 'w');
__sg_storage->append_wrapped_real(width, "width");
__sg_storage->append_wrapped_real(svm_c, "svm_c");
__sg_storage->append_wrapped_real(svr_param, "svr_param");
__sg_storage->append_wrapped_real_vector(alpha, "alpha");
__sg_storage->append_wrapped_real(mse, "mse");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
