import numpy as np
from modshogun import CHAIDTree
from modshogun import MulticlassLabels
from modshogun import WrappedObjectArray
from modshogun import SerializableAsciiFile
from modshogun import RealFeatures
from modshogun import CSVFile
from modshogun import IntVector
from modshogun import MulticlassAccuracy

f_feats_train = CSVFile("../../data/classifier_4class_2d_linear_features_train.dat")
f_feats_test = CSVFile("../../data/classifier_4class_2d_linear_features_test.dat")
f_labels_train = CSVFile("../../data/classifier_4class_2d_linear_labels_train.dat")
f_labels_test = CSVFile("../../data/classifier_4class_2d_linear_labels_test.dat")

#![create_features]
features_train = RealFeatures(f_feats_train)
features_test = RealFeatures(f_feats_test)
labels_train = MulticlassLabels(f_labels_train)
labels_test = MulticlassLabels(f_labels_test)
#![create_features]

#![set_feature_types]
ft = np.zeros( (2), dtype='int32')
ft[0] = 2
ft[1] = 2
#![set_feature_types]

#![create_instance]
classifier = CHAIDTree(0, ft, 10)
classifier.set_labels(labels_train)
#![create_instance]

#![train_and_apply]
classifier.train(features_train)
labels_predict = classifier.apply_multiclass(features_test)
#![train_and_apply]

#![evaluate_accuracy]
eval = MulticlassAccuracy()
accuracy = eval.evaluate(labels_predict, labels_test)
#![evaluate_accuracy]

# integration testing variables
output = labels_predict.get_labels()

# Serialize output for integration testing (automatically generated)
__sg_storage = WrappedObjectArray()
__sg_storage_file = SerializableAsciiFile("chaid_tree.dat", 'w')
__sg_storage.append_wrapped_real(accuracy, "accuracy")
__sg_storage.append_wrapped_real_vector(output, "output")
__sg_storage.save_serializable(__sg_storage_file)
