import numpy as np
from modshogun import MulticlassAccuracy
from modshogun import MulticlassLabels
from modshogun import SerializableAsciiFile
from modshogun import WrappedObjectArray
from modshogun import LinearMulticlassMachine
from modshogun import LibLinear
from modshogun import CSVFile
from modshogun import MulticlassOneVsOneStrategy
from modshogun import RealFeatures

f_feats_train = CSVFile("../../data/classifier_4class_2d_linear_features_train.dat")
f_feats_test = CSVFile("../../data/classifier_4class_2d_linear_features_test.dat")
f_labels_train = CSVFile("../../data/classifier_4class_2d_linear_labels_train.dat")
f_labels_test = CSVFile("../../data/classifier_4class_2d_linear_labels_test.dat")

#![create_features]
features_train = RealFeatures(f_feats_train)
features_test = RealFeatures(f_feats_test)
labels_train = MulticlassLabels(f_labels_train)
labels_test = MulticlassLabels(f_labels_test)
#![create_features]

#![create_classifier]
classifier = LibLinear()
#![create_classifier]

#![choose_strategy]
strategy = MulticlassOneVsOneStrategy()
#![choose_strategy]

#![create_instance]
mc_classifier = LinearMulticlassMachine(strategy, features_train, classifier, labels_train)
#![create_instance]

#![train_and_apply]
mc_classifier.train()
labels_predict = mc_classifier.apply_multiclass(features_test)
#![train_and_apply]

#![evaluate_accuracy]
eval = MulticlassAccuracy()
accuracy = eval.evaluate(labels_predict, labels_test)
#![evaluate_accuracy]

# integration testing variables
output = labels_predict.get_labels()

# Serialize output for integration testing (automatically generated)
__sg_storage = WrappedObjectArray()
__sg_storage_file = SerializableAsciiFile("multiclass_linearmachine.dat", 'w')
__sg_storage.append_wrapped_real(accuracy, "accuracy")
__sg_storage.append_wrapped_real_vector(output, "output")
__sg_storage.save_serializable(__sg_storage_file)
