import numpy as np
from modshogun import CSVFile
from modshogun import MulticlassLabels
from modshogun import MulticlassAccuracy
from modshogun import RealFeatures
from modshogun import MajorityVote
from modshogun import Math
from modshogun import RandomForest
from modshogun import WrappedObjectArray
from modshogun import SerializableAsciiFile

f_feats_train = CSVFile("../../data/classifier_4class_2d_linear_features_train.dat")
f_feats_test = CSVFile("../../data/classifier_4class_2d_linear_features_test.dat")
f_labels_train = CSVFile("../../data/classifier_4class_2d_linear_labels_train.dat")
f_labels_test = CSVFile("../../data/classifier_4class_2d_linear_labels_test.dat")
Math.init_random(1)

#![create_features]
features_train = RealFeatures(f_feats_train)
features_test = RealFeatures(f_feats_test)
labels_train = MulticlassLabels(f_labels_train)
labels_test = MulticlassLabels(f_labels_test)
#![create_features]

#![create_combination_rule]
m_vote = MajorityVote()
#![create_combination_rule]

#![create_instance]
rand_forest = RandomForest(features_train, labels_train, 100)
rand_forest.set_combination_rule(m_vote)
#![create_instance]

#![train_and_apply]
rand_forest.train()
labels_predict = rand_forest.apply_multiclass(features_test)
#![train_and_apply]

#![evaluate_accuracy]
acc = MulticlassAccuracy()
oob = rand_forest.get_oob_error(acc)
accuracy = acc.evaluate(labels_predict, labels_test)
#![evaluate_accuracy]

# additional integration testing variables
output = labels_predict.get_labels()


# Serialize output for integration testing (automatically generated)
__sg_storage = WrappedObjectArray()
__sg_storage_file = SerializableAsciiFile("random_forest.dat", 'w')
__sg_storage.append_wrapped_real(oob, "oob")
__sg_storage.append_wrapped_real(accuracy, "accuracy")
__sg_storage.append_wrapped_real_vector(output, "output")
__sg_storage.save_serializable(__sg_storage_file)
