import numpy as np
from modshogun import GaussianKernel
from modshogun import RelaxedTree
from modshogun import MulticlassLibLinear
from modshogun import RealFeatures
from modshogun import CSVFile
from modshogun import MulticlassLabels
from modshogun import SerializableAsciiFile
from modshogun import WrappedObjectArray
from modshogun import MulticlassAccuracy

f_feats_train = CSVFile("../../data/classifier_4class_2d_linear_features_train.dat")
f_feats_test = CSVFile("../../data/classifier_4class_2d_linear_features_test.dat")
f_labels_train = CSVFile("../../data/classifier_4class_2d_linear_labels_train.dat")
f_labels_test = CSVFile("../../data/classifier_4class_2d_linear_labels_test.dat")

#![create_features]
features_train = RealFeatures(f_feats_train)
features_test = RealFeatures(f_feats_test)
labels_train = MulticlassLabels(f_labels_train)
labels_test = MulticlassLabels(f_labels_test)
#![create_features]

#![set_parameters]
mll = MulticlassLibLinear()
kernel = GaussianKernel()
#![set_parameters]

#![create_instance]
machine = RelaxedTree()
machine.set_labels(labels_train)
machine.set_machine_for_confusion_matrix(mll)
machine.set_kernel(kernel)
#![create_instance]

#![train_and_apply]
machine.train(features_train)
labels_predict = machine.apply_multiclass(features_test)
#![train_and_apply]

#![evaluate_accuracy]
eval = MulticlassAccuracy()
accuracy = eval.evaluate(labels_predict, labels_test)
#![evaluate_accuracy]

# integration testing variables
output = labels_predict.get_labels()

# Serialize output for integration testing (automatically generated)
__sg_storage = WrappedObjectArray()
__sg_storage_file = SerializableAsciiFile("relaxed_tree.dat", 'w')
__sg_storage.append_wrapped_real(accuracy, "accuracy")
__sg_storage.append_wrapped_real_vector(output, "output")
__sg_storage.save_serializable(__sg_storage_file)
