import numpy as np
from modshogun import MeanSquaredError
from modshogun import CSVFile
from modshogun import RegressionLabels
from modshogun import SerializableAsciiFile
from modshogun import KRRNystrom
from modshogun import WrappedObjectArray
from modshogun import GaussianKernel
from modshogun import RealFeatures

f_feats_train = CSVFile("../../data/regression_1d_sinc_features_train.dat")
f_feats_test = CSVFile("../../data/regression_1d_sinc_features_test.dat")
f_labels_train = CSVFile("../../data/regression_1d_sinc_labels_train.dat")
f_labels_test = CSVFile("../../data/regression_1d_sinc_labels_test.dat")

#![create_features]
features_train = RealFeatures(f_feats_train)
features_test = RealFeatures(f_feats_test)
labels_train = RegressionLabels(f_labels_train)
labels_test = RegressionLabels(f_labels_test)
#![create_features]

#![create_appropriate_kernel]
width = 1.0
kernel = GaussianKernel(features_train, features_train, width)
#![create_appropriate_kernel]

#![create_instance]
tau = 0.001
m = 100
nystrom = KRRNystrom(tau, m, kernel, labels_train)
#![create_instance]

#![train_and_apply]
nystrom.train()
labels_predict = nystrom.apply_regression(features_test)
#![train_and_apply]

#![extract_alpha]
alpha = nystrom.get_alphas()
#![extract_alpha]

#![evaluate_error]
eval = MeanSquaredError()
mse = eval.evaluate(labels_predict, labels_test)
#![evaluate_error]

# integration testing variables
output = labels_test.get_labels()

# Serialize output for integration testing (automatically generated)
__sg_storage = WrappedObjectArray()
__sg_storage_file = SerializableAsciiFile("krr_nystrom.dat", 'w')
__sg_storage.append_wrapped_real(width, "width")
__sg_storage.append_wrapped_real(tau, "tau")
__sg_storage.append_wrapped_int(m, "m")
__sg_storage.append_wrapped_real_vector(alpha, "alpha")
__sg_storage.append_wrapped_real(mse, "mse")
__sg_storage.append_wrapped_real_vector(output, "output")
__sg_storage.save_serializable(__sg_storage_file)
