library(shogun)

f_feats_train <- CSVFile("../../data/regression_1d_linear_features_train.dat")
f_feats_test <- CSVFile("../../data/regression_1d_linear_features_test.dat")
f_labels_train <- CSVFile("../../data/regression_1d_linear_labels_train.dat")
f_labels_test <- CSVFile("../../data/regression_1d_linear_labels_test.dat")

#![create_features]
features_train <- RealFeatures(f_feats_train)
features_test <- RealFeatures(f_feats_test)
labels_train <- RegressionLabels(f_labels_train)
labels_test <- RegressionLabels(f_labels_test)
#![create_features]

#![create_instance]
tau <- 0.001
lrr <- LinearRidgeRegression(tau, features_train, labels_train)
#![create_instance]

#![train_and_apply]
lrr$train()
labels_predict <- lrr$apply_regression(features_test)
bias <- lrr$get_bias()
#![train_and_apply]

#[!disable_bias]
lrr$set_compute_bias(FALSE)
#[!disable_bias]

#[!set_bias_manually]
lrr$set_bias(bias)
#[!set_bias_manually]

#[!extract_w]
weights <- lrr$get_w()
#[!extract_w]

#![evaluate_error]
eval <- MeanSquaredError()
mse <- eval$evaluate(labels_predict, labels_test)
#![evaluate_error]

# integration testing variables
output <- labels_test$get_labels()

# Serialize output for integration testing (automatically generated)
__sg_storage <- WrappedObjectArray()
__sg_storage_file <- SerializableAsciiFile("linear_ridge_regression.dat", 'w')
__sg_storage$append_wrapped_real(tau, "tau")
__sg_storage$append_wrapped_real(bias, "bias")
__sg_storage$append_wrapped_real_vector(weights, "weights")
__sg_storage$append_wrapped_real(mse, "mse")
__sg_storage$append_wrapped_real_vector(output, "output")
__sg_storage$save_serializable(__sg_storage_file)
