require 'modshogun'

f_feats_train = Modshogun::CSVFile.new "../../data/classifier_binary_2d_linear_features_train.dat"
f_feats_test = Modshogun::CSVFile.new "../../data/classifier_binary_2d_linear_features_test.dat"
f_labels_train = Modshogun::CSVFile.new "../../data/classifier_binary_2d_linear_labels_train.dat"
f_labels_test = Modshogun::CSVFile.new "../../data/classifier_binary_2d_linear_labels_test.dat"

#![create_features]
features_train = Modshogun::RealFeatures.new f_feats_train
features_test = Modshogun::RealFeatures.new f_feats_test
labels_train = Modshogun::BinaryLabels.new f_labels_train
labels_test = Modshogun::BinaryLabels.new f_labels_test
#![create_features]

#![set_parameters]
C = 1.0
epsilon = 0.001
#![set_parameters]

#![create_instance]
svm = Modshogun::LibLinear.new C, features_train, labels_train
svm.set_liblinear_solver_type Modshogun::L2R_L2LOSS_SVC
svm.set_epsilon epsilon
#![create_instance]

#![train_and_apply]
svm.train 
labels_predict = svm.apply_binary features_test
#![train_and_apply]

#![extract_weights_bias]
w = svm.get_w 
b = svm.get_bias 
#![extract_weights_bias]

#![evaluate_accuracy]
eval = Modshogun::AccuracyMeasure.new 
accuracy = eval.evaluate labels_predict, labels_test
#![evaluate_accuracy]

# additional integration testing variables
output = labels_predict.get_labels 

# Serialize output for integration testing (automatically generated)
__sg_storage = Modshogun::WrappedObjectArray.new 
__sg_storage_file = Modshogun::SerializableAsciiFile.new "linear_svm.dat", 'w'
__sg_storage.append_wrapped_real C, "C"
__sg_storage.append_wrapped_real epsilon, "epsilon"
__sg_storage.append_wrapped_real_vector w, "w"
__sg_storage.append_wrapped_real b, "b"
__sg_storage.append_wrapped_real accuracy, "accuracy"
__sg_storage.append_wrapped_real_vector output, "output"
__sg_storage.save_serializable __sg_storage_file
